Package: formatters
Title: ASCII Formatting for Values and Tables
Version: 0.5.12
Date: 2025-12-05
Authors@R: c(
    person("Gabriel", "Becker", , "gabembecker@gmail.com", role = "aut",
           comment = "original creator of the package"),
    person("Adrian", "Waddell", , "adrian.waddell@gene.com", role = "aut"),
    person("Davide", "Garolini", , "davide.garolini@roche.com", role = "aut",
           comment = c(ORCID = "0000-0002-1445-1369")),
    person("Emily", "de la Rua", , "emily.de_la_rua@contractors.roche.com", role = "aut",
           comment = c(ORCID = "0009-0000-8738-5561")),
    person("Abinaya", "Yogasekaram", , "abinaya.yogasekaram@contractors.roche.com", role = "ctb",
           comment = c(ORCID = "0009-0005-2083-1105")),
    person("Joe", "Zhu", , "joe.zhu@roche.com", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0001-7566-2787")),
    person("F. Hoffmann-La Roche AG", role = c("cph", "fnd"))
  )
Description: We provide a framework for rendering complex tables to ASCII,
    and a set of formatters for transforming values or sets of values into
    ASCII-ready display strings.
License: Apache License 2.0
URL: https://insightsengineering.github.io/formatters/,
        https://github.com/insightsengineering/formatters/
BugReports: https://github.com/insightsengineering/formatters/issues
Depends: methods, R (>= 2.10)
Imports: checkmate (>= 2.1.0), grid, htmltools (>= 0.5.8.1), lifecycle
        (>= 0.2.0), stringi (>= 1.7.12)
Suggests: digest (>= 0.6.37), dplyr (>= 1.0.9), gt (>= 0.10.0),
        huxtable (>= 2.0.0), knitr (>= 1.50), processx (>= 3.8.4),
        r2rtf (>= 0.3.2), rmarkdown (>= 2.28), sass (>= 0.4.9),
        testthat (>= 3.2.1.1), withr (>= 2.0.0)
VignetteBuilder: knitr, rmarkdown
Config/Needs/verdepcheck: mllg/checkmate, rstudio/htmltools,
        r-lib/lifecycle, tidyverse/dplyr, rstudio/gt,
        hughjonesd/huxtable, yihui/knitr, Merck/r2rtf,
        rstudio/rmarkdown, gagolews/stringi, r-lib/testthat,
        r-lib/withr
Config/Needs/website: insightsengineering/nesttemplate
Encoding: UTF-8
Language: en-US
LazyData: true
RoxygenNote: 7.3.3
Collate: 'data.R' 'format_value.R' 'matrix_form.R' 'generics.R'
        'labels.R' 'mpf_exporters.R' 'package.R' 'page_size.R'
        'pagination.R' 'tostring.R' 'utils.R' 'zzz.R'
NeedsCompilation: no
Packaged: 2025-12-06 01:03:54 UTC; rstudio
Author: Gabriel Becker [aut] (original creator of the package),
  Adrian Waddell [aut],
  Davide Garolini [aut] (ORCID: <https://orcid.org/0000-0002-1445-1369>),
  Emily de la Rua [aut] (ORCID: <https://orcid.org/0009-0000-8738-5561>),
  Abinaya Yogasekaram [ctb] (ORCID:
    <https://orcid.org/0009-0005-2083-1105>),
  Joe Zhu [aut, cre] (ORCID: <https://orcid.org/0000-0001-7566-2787>),
  F. Hoffmann-La Roche AG [cph, fnd]
Maintainer: Joe Zhu <joe.zhu@roche.com>
Repository: CRAN
Date/Publication: 2025-12-08 10:00:02 UTC
Built: R 4.6.0; ; 2026-04-27 06:36:05 UTC; unix
