# Generated by roxygen2: do not edit by hand

export("%||%")
export("main_footer<-")
export("main_title<-")
export("mf_aligns<-")
export("mf_cinfo<-")
export("mf_colgap<-")
export("mf_display<-")
export("mf_fontspec<-")
export("mf_formats<-")
export("mf_lgrouping<-")
export("mf_ncol<-")
export("mf_nrheader<-")
export("mf_rfnotes<-")
export("mf_rinfo<-")
export("mf_spans<-")
export("mf_strings<-")
export("num_rep_cols<-")
export("obj_align<-")
export("obj_format<-")
export("obj_label<-")
export("obj_na_str<-")
export("obj_name<-")
export("obj_round_type<-")
export("page_titles<-")
export("prov_footer<-")
export("subtitles<-")
export("table_inset<-")
export("var_labels<-")
export(MatrixPrintForm)
export(all_footers)
export(all_titles)
export(basic_listing_mf)
export(basic_matrix_form)
export(basic_pagdf)
export(check_aligns)
export(close_font_dev)
export(debug_font_dev)
export(decimal_align)
export(default_hsep)
export(default_page_number)
export(diagnose_pagination)
export(divider_height)
export(do_forced_paginate)
export(export_as_pdf)
export(export_as_rtf)
export(export_as_txt)
export(fmt_config)
export(font_spec)
export(format_value)
export(is.wholenumber)
export(is_valid_format)
export(list_valid_aligns)
export(list_valid_format_labels)
export(main_footer)
export(main_title)
export(make_row_df)
export(matrix_form)
export(mf_aligns)
export(mf_cinfo)
export(mf_colgap)
export(mf_display)
export(mf_fontspec)
export(mf_formats)
export(mf_has_rlabels)
export(mf_has_topleft)
export(mf_lgrouping)
export(mf_ncol)
export(mf_nlheader)
export(mf_nrheader)
export(mf_nrow)
export(mf_rfnotes)
export(mf_rinfo)
export(mf_spans)
export(mf_strings)
export(mpf_has_rlabels)
export(mpf_to_rtf)
export(nchar_ttype)
export(nlines)
export(num_rep_cols)
export(obj_align)
export(obj_format)
export(obj_label)
export(obj_na_str)
export(obj_name)
export(obj_round_type)
export(open_font_dev)
export(padstr)
export(pag_indices_inner)
export(pagdfrow)
export(page_dim)
export(page_lcpp)
export(page_titles)
export(page_types)
export(paginate_indices)
export(paginate_to_mpfs)
export(propose_column_widths)
export(prov_footer)
export(ref_df_row)
export(round_fmt)
export(set_default_hsep)
export(set_default_page_number)
export(spans_to_viscell)
export(split_word_ttype)
export(spread_integer)
export(sprintf_format)
export(subtitles)
export(table_inset)
export(toString)
export(undebug_font_dev)
export(valid_round_type)
export(var_labels)
export(var_labels_remove)
export(var_relabel)
export(vert_pag_indices)
export(with_label)
export(wrap_string)
export(wrap_string_ttype)
export(wrap_txt)
exportClasses(MatrixPrintForm)
exportMethods("main_footer<-")
exportMethods("main_title<-")
exportMethods("num_rep_cols<-")
exportMethods("obj_align<-")
exportMethods("obj_format<-")
exportMethods("obj_label<-")
exportMethods("obj_na_str<-")
exportMethods("obj_round_type<-")
exportMethods("page_titles<-")
exportMethods("prov_footer<-")
exportMethods("subtitles<-")
exportMethods("table_inset<-")
exportMethods(divider_height)
exportMethods(do_forced_paginate)
exportMethods(main_footer)
exportMethods(main_title)
exportMethods(matrix_form)
exportMethods(ncol)
exportMethods(nlines)
exportMethods(num_rep_cols)
exportMethods(obj_align)
exportMethods(obj_format)
exportMethods(obj_label)
exportMethods(obj_na_str)
exportMethods(obj_round_type)
exportMethods(page_titles)
exportMethods(prov_footer)
exportMethods(subtitles)
exportMethods(table_inset)
exportMethods(toString)
import(checkmate)
import(grDevices)
import(grid)
import(methods)
importFrom(grDevices,dev.off)
importFrom(grDevices,pdf)
importFrom(grid,convertHeight)
importFrom(grid,convertWidth)
importFrom(grid,get.gpar)
importFrom(grid,gpar)
importFrom(grid,grid.draw)
importFrom(grid,grid.newpage)
importFrom(grid,grobHeight)
importFrom(grid,grobWidth)
importFrom(grid,plotViewport)
importFrom(grid,pushViewport)
importFrom(grid,textGrob)
importFrom(grid,unit)
importFrom(htmltools,tagList)
importFrom(htmltools,tags)
importFrom(lifecycle,deprecated)
importFrom(stats,na.omit)
importFrom(stringi,stri_detect)
importFrom(stringi,stri_replace)
importFrom(stringi,stri_sub)
importFrom(stringi,stri_wrap)
importFrom(tools,file_ext)
importFrom(utils,capture.output)
importFrom(utils,head)
importFrom(utils,localeToCharset)
importFrom(utils,tail)
