---
name: Bug report
about: Create a report to help us improve
title: ''
labels: 'bug'
assignees: ''

---
<!--
**Thank you for wanting to report a bug!**


⚠
Verify first that your issue is not already reported on
../search?q=is%3Aissue&type=issues.

Also test if the latest main branch are affected too.
-->


**Summary**
<!--
Please briefly describe your problem and, when relevant, the output you expect.
Please also provide the output of `utils::sessionInfo()` or
`devtools::session_info()` at the end of your post.

If at all possible, please include a minimal, reproducible
example https://stackoverflow.com/questions/5963269/how-to-make-a-great-r-reproducible-example.
-->

Your brief description of the problem

```r

# your reproducible example here

```

**R session info**
<!--
Paste verbatim output from `R -e "utils::sessionInfo()"` below.
-->

```sh

# R -e "utils::sessionInfo()" output goes here

```

**OS / Environment**
<!--
Provide all relevant information below, e.g. target OS versions, docker image, etc.
-->

- OS: [e.g. Windows 10, Ubuntu 20.04, Centos 8]
- Docker Image [e.g. rocker/verse:4.1.2]
