---
name: Feature request
about: Suggest an idea for this project
title: ''
labels: 'enhancement'
assignees: ''

---

**Summary**
<!--
Describe the new feature/improvement you would like briefly below.


What's the problem this feature will solve?

What are you trying to do, that you are unable to achieve
with this package as it currently stands?


* Provide examples of real-world use cases that this would enable
and how it solves the problem you described.

* How do you solve this now?

* Have you tried to work around the problem using other tools?

* Could there be a different approach to solving this issue?
-->

As a [persona], I [want to], [so that].


**Additional Information**
<!--
Describe how the feature would be used, why it is needed and what it would solve.

**HINT:** You can paste https://gist.github.com links for larger files.
-->