% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{ex_adsl}
\alias{ex_adsl}
\alias{ex_adae}
\alias{ex_adaette}
\alias{ex_adtte}
\alias{ex_adcm}
\alias{ex_adlb}
\alias{ex_admh}
\alias{ex_adqs}
\alias{ex_adrs}
\alias{ex_advs}
\title{Simulated CDISC Alike Data for Examples}
\format{
rds (data.frame)

An object of class \code{tbl_df} (inherits from \code{tbl}, \code{data.frame}) with 1934 rows and 48 columns.

An object of class \code{tbl_df} (inherits from \code{tbl}, \code{data.frame}) with 1200 rows and 42 columns.

An object of class \code{tbl_df} (inherits from \code{tbl}, \code{data.frame}) with 1200 rows and 42 columns.

An object of class \code{tbl_df} (inherits from \code{tbl}, \code{data.frame}) with 1934 rows and 41 columns.

An object of class \code{tbl_df} (inherits from \code{tbl}, \code{data.frame}) with 8400 rows and 59 columns.

An object of class \code{tbl_df} (inherits from \code{tbl}, \code{data.frame}) with 1934 rows and 41 columns.

An object of class \code{tbl_df} (inherits from \code{tbl}, \code{data.frame}) with 14000 rows and 49 columns.

An object of class \code{tbl_df} (inherits from \code{tbl}, \code{data.frame}) with 2400 rows and 41 columns.

An object of class \code{tbl_df} (inherits from \code{tbl}, \code{data.frame}) with 16800 rows and 59 columns.
}
\usage{
ex_adsl

ex_adae

ex_adaette

ex_adtte

ex_adcm

ex_adlb

ex_admh

ex_adqs

ex_adrs

ex_advs
}
\description{
Simulated CDISC Alike Data for Examples
}
\keyword{datasets}
