% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/generics.R
\name{print,ANY-method}
\alias{print,ANY-method}
\title{Print}
\usage{
\S4method{print}{ANY}(x, ...)
}
\arguments{
\item{x}{an object used to select a method.}

\item{...}{further arguments passed to or from other methods.}
}
\description{
Print an R object. see \code{[base::print()]}
}
