% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/format_value.R
\name{is_valid_format}
\alias{is_valid_format}
\title{Check if a format is supported}
\usage{
is_valid_format(x, stop_otherwise = FALSE)
}
\arguments{
\item{x}{either format string or an object returned by \code{sprintf_format}}

\item{stop_otherwise}{logical, if \code{x} is not a format should an error be
thrown}
}
\value{
\code{TRUE} if \code{x} is \code{NULL}, a supported format string, or a function; \code{FALSE} otherwise.
}
\description{
Check if a format is supported
}
\note{
No check if the function is actually a formatter is performed.
}
\examples{
is_valid_format("xx.x")
is_valid_format("fakeyfake")
}
