% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{DM}
\alias{DM}
\title{DM data}
\format{
rds (data.frame)
}
\usage{
DM
}
\description{
DM data
}
\keyword{datasets}
