% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/labels.R
\name{var_labels_remove}
\alias{var_labels_remove}
\title{Remove Variable Labels of a \code{data.frame}}
\usage{
var_labels_remove(x)
}
\arguments{
\item{x}{a \code{data.frame} object}
}
\value{
the same data frame as \code{x} stripped of variable labels
}
\description{
Removing labels attributes from a variables in a data frame
}
\examples{
x <- var_labels_remove(iris)
}
